var DemoRoom = Class.create();

DemoRoom.prototype = {
	initialize : function(title, videoUrl, options) {
		this.title = title;
		this.videoUrl = videoUrl;
		this.options = {videoId: "video", notesId: "notes", videoListId: "videos"};
		this.options = Object.extend(this.options, options || {});
		this.notes = [];
		this.sections = [];
	},


	render: function() {
		var menuItem = $jq('<li><a href="#">' + this.title + '</a><ul class="sections"></ul></li>');
		this.menuItem = menuItem;
		var that = this;
		menuItem.children("a").click(function() {
			$jq('#' + that.options.videoListId + ' .sections').empty();
			var sections = menuItem.children(".sections");
			that.renderSections(sections);
			that.start();
			return false;
		});
		menuItem.appendTo("#" + this.options.videoListId);
	},

	renderSections: function(domNode) {
		domNode = $jq(domNode);
		for(var i=0; i < this.sections.length; i++) {
			var that = this;
			(function(timestamp, section) {
				var li = $jq('<li>' + section + '</li>');
				li.click(function() {
					that.seek(timestamp);
				});
				domNode.append(li);
			})(this.sections[i].timestamp, this.sections[i].section);
		}
	},

	addNote: function(timestamp, note, options) {
		options = Object.extend({halt: true}, options  || {});
		this.notes.push({timestamp: timestamp * 600, note: note, options: options});
	},

	addSection: function(timestamp, section) {
		this.sections.push({timestamp: timestamp * 600, section: section});
	},
	
	start: function() {
		if(DemoRoom.currentlyRunning) {
			DemoRoom.currentlyRunning.stop();
		}
		DemoRoom.currentlyRunning = this;
		this.lastNote = null;
		
		var autoplay = true;
		if(this.notes[0] && this.notes[0].timestamp == 0 && this.notes[0].options.halt) {
			autoplay = false;
		}

		// Create player
		document.getElementById("video-display").innerHTML = '<object classid="clsid:02BF25D5-8C17-4B23-BC80-D3488ABDDC6B" codebase="http://www.apple.com/qtactivex/qtplugin.cab" width="640" height="495" id="video" align="middle">		<param name="controller" value="true" />		<param name="src" value="' + this.videoUrl + '" />		<param name="autoplay" value="' + autoplay + '" />		<param name="loop" value="false" />		<embed src="' + this.videoUrl + '" loop="false" autoplay="' + autoplay + '" controller="true" width="640" height="500" name="video" align="middle" pluginspage="http://www.apple.com/quicktime/" EnableJavaSript="true" />		</object>';
		this.timer = setTimeout(this._start.bindAsEventListener(this), 500);
	},
	
	_start: function() {
		this.videoControl = document[this.options.videoId];
		
		try {
			this.videoControl.GetTime();
		} catch(e) {
			this.timer = setTimeout(this._start.bindAsEventListener(this), 1000);
			return;
		}

		this.notesDisplay = $jq("#" + this.options.notesId);
		this.videoDisplay = $jq("#" + this.options.videoId);
		this.currentFrameIndex = 0;

		this.videoDisplay.show();
		this.timer = setTimeout(this.resume.bindAsEventListener(this), 1000);
	},
	
	stop: function() {
		clearTimeout(this.timer);
	},
	
	seek: function(timestamp) {
		try {
			this.videoControl.SetTime(timestamp);
			if(this.videoControl.GetTime() < timestamp) {
				alert("The video hasn't loaded this far yet, try waiting a bit and try again.");
			}
		} catch(e) {}		
	},
	
	resume: function() {
		try {
			this.videoControl.Play();			
		} catch(e) {}
		var nextNote = this.notes[this.currentFrameIndex];
		this.timer = setTimeout(this.frame.bindAsEventListener(this), 200);
	},
	
	frame: function() {
		this.timer = setTimeout(this.frame.bindAsEventListener(this), 200);
		try {
			var currentTime = this.videoControl.GetTime();			
		} catch(e) {
			return;
		}
		
		$jq("#log").empty();
		$jq("#log").append("<br />Current Time: " + currentTime / 600);
		var currentNote;
		var currentSection;
		for(var i=0; i < this.notes.length; i++) {
			if(this.notes[i].timestamp <= currentTime) {
				currentNote = this.notes[i];
			} else {
				break;
			}
		}

		for(currentSection=0; currentSection < this.sections.length; currentSection++) {
			if(this.sections[currentSection].timestamp > currentTime) {
				break;
			}
		}

		currentSection--;
		
		if(currentSection != this.lastSection) {
			var sectionElements = $jq(this.menuItem).children(".sections").children("li");
			sectionElements.removeClass("current");
			$jq(sectionElements[currentSection]).addClass("current");
			this.lastSection = currentSection;
		}
		
		if(!currentNote || currentNote == this.lastNote) {
			return;
		}
		
		if(currentNote.options.halt) {
			this.videoControl.Stop();
		}
		
		this.displayNote(currentNote);
		this.lastNote = currentNote;
	},
		
	displayNote: function(note) {
		this.notesDisplay.empty();
		this.notesDisplay.append(note.note);
		if(note.options.halt) {
			var resumeButton = document.createElement("button");
			resumeButton.innerHTML = "Continue";
			resumeButton.className = "continue";
			that = this;
			resumeButton.onclick = function() {
				that.resume();
				$jq(this).hide();
			}
			this.notesDisplay.append(resumeButton);
		}
		// this.notesDisplay.fadeIn("fast");
		new Effect.Highlight(this.notesDisplay[0], {startcolor: "#ffffff"});
	}
};